----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    11:03:33 03/01/2015 
-- Module Name:    KeyInvKeyExpansion - Behavioral 
-- Project Name:   POET 2
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE ieee.numeric_std.ALL;

entity KeyInvKeyExpansion is
    Port ( input   : in  STD_LOGIC_VECTOR (127 downto 0);
			  encrypt : in  STD_LOGIC;
			  Rcon    : in  STD_LOGIC_VECTOR (7 downto 0);
			  output  : out STD_LOGIC_VECTOR (127 downto 0));			  
end KeyInvKeyExpansion;

architecture Behavioral of KeyInvKeyExpansion is
	
	COMPONENT KeySchedCore is
   Port (  input 		: in  STD_LOGIC_VECTOR (31 downto 0);
           Rcon 		: in  STD_LOGIC_VECTOR (7 downto 0);
			  output 	: out STD_LOGIC_VECTOR (31 downto 0));
	end COMPONENT;

	type WordType is array(0 to 3) of std_logic_vector(31 downto 0);
		
	signal	InWords				: WordType;
	signal	ProcessedInWords	: WordType;
	signal 	OutWords 			: WordType;
	signal	tmp					: STD_LOGIC_VECTOR(31 downto 0);
	
begin
	
	GenInWords:
	for i in 0 to 3 generate
			InWords(i) <= input(((4-i)*32-1) downto ((3-i)*32));
	end generate GenInWords;

	ProcessedInWords(3)	<= InWords(3) WHEN encrypt = '1' ELSE OutWords(3);
		
	KeySchedCoreInst : KeySchedCore
	Port map(
		input		=> ProcessedInWords(3),
		Rcon		=> Rcon,
		output	=> tmp);
	
	OutWords(0) 			<= InWords(0) xor tmp;
	
	ProcessedInWords(0)	<= OutWords(0) WHEN encrypt = '1' ELSE InWords(0);
	OutWords(1) 			<= ProcessedInWords(0) xor InWords(1); 
	
	ProcessedInWords(1)	<= OutWords(1) WHEN encrypt = '1' ELSE InWords(1);
	OutWords(2) 			<= ProcessedInWords(1) xor InWords(2); 
	
	ProcessedInWords(2)	<= OutWords(2) WHEN encrypt = '1' ELSE InWords(2);
	OutWords(3) 			<= ProcessedInWords(2) xor InWords(3); 
		
	GenOutWords:
	for i in 0 to 3 generate
		output(((4-i)*32-1) downto ((3-i)*32)) <= OutWords(i);
	end generate GenOutWords;
	
end Behavioral;

