----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    14:05:48 01/01/2016 
-- Module Name:    MainModule - Behavioral 
-- Project Name:   POET 2
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity MainModule is
    Port (  input     					: in  STD_LOGIC_VECTOR (127 downto 0);
            clk							: in  STD_LOGIC;
				ForcedInputBits	 		: in  STD_LOGIC_VECTOR (3 downto 0);
				SaveK      					: in  STD_LOGIC;
				SaveK10    					: in  STD_LOGIC;
				SaveL      					: in  STD_LOGIC;
				SaveKF  						: in  STD_LOGIC;
				SaveTau    					: in  STD_LOGIC;
				ProcessLStart				: in  STD_LOGIC;
				ProcessLEnable				: in  STD_LOGIC;
				ProcessLClear				: in  STD_LOGIC;
				BlockCounterEnable		: in  STD_LOGIC;
				BlockCounterReset			: in  STD_LOGIC;
				InputOrMLength				: in  STD_LOGIC;
				AEStopOrL					: in  STD_LOGIC;
				KOrSK							: in  STD_LOGIC;
				AESMainReset				: in  STD_LOGIC;
				AESMainEncrypt				: in  STD_LOGIC;
				StartXY						: in  STD_LOGIC;
				AEStopReset					: in  STD_LOGIC;
				AESbotReset					: in  STD_LOGIC;
				AESbotOrFeedback			: in  STD_LOGIC;
				OutputOrTau					: in  STD_LOGIC;
				FeedbackRegClear			: in  STD_LOGIC;
				FeedbackRegEnable			: in  STD_LOGIC;
				LoopEnable					: in  STD_LOGIC;
				AESMainDone					: out STD_LOGIC;
				AEStopDone					: out STD_LOGIC;
				AESbotDone					: out STD_LOGIC;
            output 						: out STD_LOGIC_VECTOR (127 downto 0));
end MainModule;

architecture Behavioral of MainModule is

	COMPONENT reg_FDE IS
	GENERIC ( size: POSITIVE := 128);
	PORT ( clk  : IN  STD_LOGIC;
			 D    : IN  STD_LOGIC_VECTOR ((size-1) DOWNTO 0);
			 en   : IN  STD_LOGIC;
			 Q    : OUT STD_LOGIC_VECTOR ((size-1) DOWNTO 0));
	END COMPONENT;

	COMPONENT Process_L is
   PORT ( input 	: in  STD_LOGIC_VECTOR (127 downto 0);
          clk 		: in  STD_LOGIC;
          start 	: in  STD_LOGIC;
          enable 	: in  STD_LOGIC;
          clear 	: in  STD_LOGIC;
          output 	: out  STD_LOGIC_VECTOR (127 downto 0));
	end COMPONENT;

	COMPONENT counter IS
	GENERIC ( size: POSITIVE);
	PORT ( clk     : IN  STD_LOGIC;
			 en      : IN  STD_LOGIC;
			 reset	: IN  STD_LOGIC;
			 q 		: OUT STD_LOGIC_VECTOR ((size-1) DOWNTO 0));
	END COMPONENT;
	
	COMPONENT AES_Encyption is
	Port ( input  : in  STD_LOGIC_VECTOR (127 downto 0);
		    key    : in  STD_LOGIC_VECTOR (127 downto 0);
			 reset  : in  STD_LOGIC;
			 clk    : in  STD_LOGIC;
			 output : out STD_LOGIC_VECTOR (127 downto 0);
			 done   : out STD_LOGIC);
	end COMPONENT;

	COMPONENT AES_EncyptionDecryption is
   Port ( input  			: in  STD_LOGIC_VECTOR (127 downto 0);
          key    			: in  STD_LOGIC_VECTOR (127 downto 0);
          reset  			: in  STD_LOGIC;
          clk  		  	: in  STD_LOGIC;
			 encrypt			: in  STD_LOGIC;
          output 			: out STD_LOGIC_VECTOR (127 downto 0);
			 roundkeyout	: out STD_LOGIC_VECTOR (127 downto 0);
          done   			: out STD_LOGIC);
	end COMPONENT;

	COMPONENT reg_FDER IS
	GENERIC ( size: POSITIVE := 128);
	PORT ( clk  : IN  STD_LOGIC;
			 D    : IN  STD_LOGIC_VECTOR ((size-1) DOWNTO 0);
			 clr  : IN  STD_LOGIC;
			 en   : IN  STD_LOGIC;
			 Q    : OUT STD_LOGIC_VECTOR ((size-1) DOWNTO 0));
	END COMPONENT;

	------------------------------------------------

	signal K							: STD_LOGIC_VECTOR (127 downto 0);
	signal K10						: STD_LOGIC_VECTOR (127 downto 0);
	signal L							: STD_LOGIC_VECTOR (127 downto 0);
	signal KF						: STD_LOGIC_VECTOR (127 downto 0);
   signal TauReg					: STD_LOGIC_VECTOR (127 downto 0);

   signal ProcessLOutput		: STD_LOGIC_VECTOR (127 downto 0);
	
	signal BlockCounterValue	: STD_LOGIC_VECTOR (11 downto 0);

   signal MLength					: STD_LOGIC_VECTOR (127 downto 0);
   signal FunctionPreInput		: STD_LOGIC_VECTOR (127 downto 0);
   signal FunctionInput			: STD_LOGIC_VECTOR (127 downto 0);
   signal SecondFeedback		: STD_LOGIC_VECTOR (127 downto 0);
   signal SelectedKOrK10		: STD_LOGIC_VECTOR (127 downto 0);
   signal AESMainInput			: STD_LOGIC_VECTOR (127 downto 0);
   signal AESMainKey				: STD_LOGIC_VECTOR (127 downto 0);
	signal AESMainOutput			: STD_LOGIC_VECTOR (127 downto 0);
	signal AESMainKeyOut			: STD_LOGIC_VECTOR (127 downto 0);

	signal TauXORENC				: STD_LOGIC_VECTOR (127 downto 0);
	signal TauXORDEC				: STD_LOGIC_VECTOR (127 downto 0);

	signal AEStopInput			: STD_LOGIC_VECTOR (127 downto 0);
	signal AEStopOutput			: STD_LOGIC_VECTOR (127 downto 0);
	
	signal AESbotInput			: STD_LOGIC_VECTOR (127 downto 0);
	signal AESbotOutput			: STD_LOGIC_VECTOR (127 downto 0);

	signal FirstAdd				: STD_LOGIC_VECTOR (127 downto 0);
	signal SecondAdd				: STD_LOGIC_VECTOR (127 downto 0);
	signal FeedbackRegInput		: STD_LOGIC_VECTOR (127 downto 0);
	signal Feedback				: STD_LOGIC_VECTOR (127 downto 0);
	signal FeedbackChecked		: STD_LOGIC_VECTOR (127 downto 0);

begin
	
	KReg: reg_FDE
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> AESMainOutput,
		en		=> SaveK,
		Q		=> K);	
	
	K10Reg: reg_FDE
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> AESMainKeyOut,
		en		=> SaveK10,
		Q		=> K10);	

	LReg: reg_FDE
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> AESMainOutput,
		en		=> SaveL,
		Q		=> L);	
	
	KFReg: reg_FDE
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> AESMainOutput,
		en		=> SaveKF,
		Q		=> KF);	
	
	TauRegIns: reg_FDE
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> AESMainOutput,
		en		=> SaveTau,
		Q		=> TauReg);	

	--------------------------------------------
	
	Process_L_Ins: Process_L 
   PORT MAP( 
		input 	=> L,
      clk 		=> clk,
      start 	=> ProcessLStart,
      enable 	=> ProcessLEnable,
      clear 	=> ProcessLClear,
      output 	=> ProcessLOutput);

	--------------------------------------------

	CounterIns: counter
	GENERIC MAP ( size => 12)
	PORT MAP ( 
		clk   => clk,
		en    => BlockCounterEnable,
		reset	=> BlockCounterReset,
		q 		=> BlockCounterValue);

	MLength					<= (BlockCounterValue(0) & "0000" & "000") & BlockCounterValue(8 downto 1) & ("00000" & BlockCounterValue(11 downto 9)) & x"00000000000000000000000000";
	FunctionPreInput		<= input WHEN InputOrMLength = '0' ELSE MLength;
	FunctionInput			<= (FunctionPreInput(127) OR ForcedInputBits(3)) &  FunctionPreInput(126 downto 64) & (FunctionPreInput(63) OR ForcedInputBits(2)) & FunctionPreInput(62 downto 2) & (FunctionPreInput(1 downto 0) OR ForcedInputBits(1 downto 0));
	
	SecondFeedback			<= AEStopOutput WHEN AEStopOrL = '0' ELSE ProcessLOutput;
	AESMainInput			<= FunctionInput XOR FeedbackChecked XOR SecondFeedback;
	
	SelectedKOrK10			<= K 					WHEN AESMainEncrypt = '1' ELSE K10;
	AESMainKey				<= SelectedKOrK10 WHEN KOrSK  = '0' ELSE input;	
	
	AESMain: AES_EncyptionDecryption
	PORT MAP (
		input   		=> AESMainInput,
		key    		=> AESMainKey,
		reset 		=> AESMainReset,
		clk  		   => clk,
		encrypt 		=> AESMainEncrypt,
		output  		=> AESMainOutput,
		roundkeyout	=> AESMainKeyOut,
		done    		=> AESMainDone);

	-------
		
	TauXORENC      <=	TauReg(127 downto 1) & (TauReg(0) XOR  AESMainEncrypt);
	TauXORDEC      <=	TauReg(127 downto 1) & (TauReg(0) XNOR AESMainEncrypt);
		
	AEStopInput		<= TauXORDEC WHEN StartXY = '1' ELSE AESMainInput;
	
	AEStop: AES_Encyption 
	PORT MAP (
		input  => AEStopInput,
		key    => KF,
		reset  => AEStopReset,
		clk    => clk,
		output => AEStopOutput,
		done   => AEStopDone);
	
	-------
		
	AESbotInput		<= TauXORENC WHEN StartXY = '1' ELSE AESMainOutput;

	AESbot: AES_Encyption 
	PORT MAP (
		input  => AESbotInput,
		key    => KF,
		reset  => AESbotReset,
		clk    => clk,
		output => AESbotOutput,
		done   => AESbotDone);
	
	-----------------------
	
	FirstAdd				<= AESbotOutput WHEN AESbotOrFeedback = '0' ELSE Feedback;
	SecondAdd			<= AESMainOutput XOR FirstAdd;
	FeedbackRegInput	<= SecondAdd WHEN OutputOrTau = '0' ELSE TauReg;
	
	FeedbackReg: reg_FDER
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> FeedbackRegInput,
		clr	=> FeedbackRegClear,
		en		=> FeedbackRegEnable,
		Q		=> Feedback);	

	FeedbackChecked	<= Feedback WHEN LoopEnable = '1' ELSE x"00000000000000000000000000000000";
	output				<= Feedback XOR SecondAdd;

	------------------------------------------------------

end Behavioral;

