----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    14:00:47 03/01/2015  
-- Module Name:    MixInvMixOneColumn - Behavioral 
-- Project Name:   POET 2
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity MixInvMixOneColumn is
    Port ( input   : in  STD_LOGIC_VECTOR (31 downto 0);
			  encrypt : in  STD_LOGIC;
           output  : out STD_LOGIC_VECTOR (31 downto 0));
end MixInvMixOneColumn;

architecture Behavioral of MixInvMixOneColumn is
	
	COMPONENT Mul2 is
			Port ( input  : in  STD_LOGIC_VECTOR (7 downto 0);
					 output : out STD_LOGIC_VECTOR (7 downto 0));
	end COMPONENT;	
	
	type 		ByteType is array(0 to 3) of std_logic_vector(7 downto 0);
	signal	InBytes    					: ByteType;
	signal	ProcessedInput			 	: ByteType;
	signal	DoubleBytes					: ByteType;
	signal 	OutBytes   					: ByteType;
	
	signal	Input0XOR2					: STD_LOGIC_VECTOR(7 downto 0);
	signal	Input1XOR3					: STD_LOGIC_VECTOR(7 downto 0);

	signal	Double0XOR2					: STD_LOGIC_VECTOR(7 downto 0);
	signal	Double1XOR3					: STD_LOGIC_VECTOR(7 downto 0);

	signal	Quadruple0XOR2				: STD_LOGIC_VECTOR(7 downto 0);
	signal	Quadruple1XOR3				: STD_LOGIC_VECTOR(7 downto 0);
	signal	Quadruple0XOR2Checked	: STD_LOGIC_VECTOR(7 downto 0);
	signal	Quadruple1XOR3Checked	: STD_LOGIC_VECTOR(7 downto 0);
	
begin

	GenInBytes:
	for i in 0 to 3 generate
			InBytes(i) <= input(((4-i)*8-1) downto ((3-i)*8));
	end generate GenInBytes;
	
	-- process needed for InvMixColumn ----------------

	Input0XOR2		<= InBytes(0) XOR InBytes(2);
	Input1XOR3		<= InBytes(1) XOR InBytes(3);
	
	Mul2Inst1: Mul2
	port map(
		input    => Input0XOR2,	
		output   => Double0XOR2
		);
	
	Mul2Inst2: Mul2
	port map(
		input    => Input1XOR3,	
		output   => Double1XOR3
		);
	
	Mul2Inst3: Mul2
	port map(
		input    => Double0XOR2,	
		output   => Quadruple0XOR2
		);
	
	Mul2Inst4: Mul2
	port map(
		input    => Double1XOR3,	
		output   => Quadruple1XOR3
		);
		
	Quadruple0XOR2Checked	<= Quadruple0XOR2 WHEN encrypt = '0' ELSE x"00";
	Quadruple1XOR3Checked	<= Quadruple1XOR3 WHEN encrypt = '0' ELSE x"00";
	
	-- process needed for InvMixColumn done ----------------
	
	ProcessedInput(0)			<= InBytes(0) XOR Quadruple0XOR2Checked;
	ProcessedInput(1)			<= InBytes(1) XOR Quadruple1XOR3Checked;
	ProcessedInput(2)			<= InBytes(2) XOR Quadruple0XOR2Checked;
	ProcessedInput(3)			<= InBytes(3) XOR Quadruple1XOR3Checked;

	GenMul2:
	for i in 0 to 3 generate
		Mul2Inst: Mul2
		port map(
			input    => ProcessedInput(i),	
			output   => DoubleBytes(i)
			);
	end generate GenMul2;
	
	CalculateOutBytes:
	for i in 0 to 3 generate
		OutBytes(i) <= DoubleBytes(i) xor DoubleBytes((i+1) mod 4) xor ProcessedInput((i+1) mod 4) xor ProcessedInput((i+2) mod 4) xor ProcessedInput((i+3) mod 4);
	end generate CalculateOutBytes;
	
	GenOutBytes:
	for i in 0 to 3 generate
			output(((4-i)*8-1) downto ((3-i)*8)) <= OutBytes(i);
	end generate GenOutBytes;

end Behavioral;

