----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    11:17:48 01/01/2016 
-- Module Name:    POET_Full - Behavioral 
-- Project Name:   POET 2
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity POET_Full is
    Port ( input 		: in  STD_LOGIC_VECTOR (127 downto 0);
           last 		: in  STD_LOGIC;
           clk			: in  STD_LOGIC;
			  ls			: in  STD_LOGIC;
			  lt			: in	STD_LOGIC;
			  ready 		: out STD_LOGIC;
           reset 		: in  STD_LOGIC;
           setkey 	: in  STD_LOGIC;
           header 	: in  STD_LOGIC;
           encrypt 	: in  STD_LOGIC;
			  decrypt 	: in  STD_LOGIC;
           output 	: out STD_LOGIC_VECTOR (127 downto 0));
end POET_Full;

architecture Behavioral of POET_Full is

	COMPONENT MainModule is
   PORT  (  input     					: in  STD_LOGIC_VECTOR (127 downto 0);
            clk							: in  STD_LOGIC;
				ForcedInputBits	 		: in  STD_LOGIC_VECTOR (3 downto 0);
				SaveK      					: in  STD_LOGIC;
				SaveK10    					: in  STD_LOGIC;
				SaveL      					: in  STD_LOGIC;
				SaveKF  						: in  STD_LOGIC;
				SaveTau    					: in  STD_LOGIC;
				ProcessLStart				: in  STD_LOGIC;
				ProcessLEnable				: in  STD_LOGIC;
				ProcessLClear				: in  STD_LOGIC;
				BlockCounterEnable		: in  STD_LOGIC;
				BlockCounterReset			: in  STD_LOGIC;
				InputOrMLength				: in  STD_LOGIC;
				AEStopOrL					: in  STD_LOGIC;
				KOrSK							: in  STD_LOGIC;
				AESMainReset				: in  STD_LOGIC;
				AESMainEncrypt				: in  STD_LOGIC;
				StartXY						: in  STD_LOGIC;
				AEStopReset					: in  STD_LOGIC;
				AESbotReset					: in  STD_LOGIC;
				AESbotOrFeedback			: in  STD_LOGIC;
				OutputOrTau					: in  STD_LOGIC;
				FeedbackRegClear			: in  STD_LOGIC;
				FeedbackRegEnable			: in  STD_LOGIC;
				LoopEnable					: in  STD_LOGIC;
				AESMainDone					: out STD_LOGIC;
				AEStopDone					: out STD_LOGIC;
				AESbotDone					: out STD_LOGIC;
            output 						: out STD_LOGIC_VECTOR (127 downto 0));
	end COMPONENT;

	COMPONENT counter IS
	GENERIC ( size: POSITIVE);
	PORT ( clk     : IN  STD_LOGIC;
			 en      : IN  STD_LOGIC;
			 reset	: IN  STD_LOGIC;
			 q 		: OUT STD_LOGIC_VECTOR ((size-1) DOWNTO 0));
	END COMPONENT;
	---------------------------------------------
	
	signal ForcedInputBits	 		: STD_LOGIC_VECTOR (3 downto 0);
	signal SaveK      				: STD_LOGIC;
	signal SaveK10    				: STD_LOGIC;
	signal SaveL      				: STD_LOGIC;
	signal SaveKF  					: STD_LOGIC;
	signal SaveTau    				: STD_LOGIC;
	signal ProcessLStart				: STD_LOGIC;
	signal ProcessLEnable			: STD_LOGIC;
	signal ProcessLClear				: STD_LOGIC;
	signal BlockCounterEnable		: STD_LOGIC;
	signal BlockCounterReset		: STD_LOGIC;
	signal InputOrMLength			: STD_LOGIC;
	signal AEStopOrL					: STD_LOGIC;
	signal KOrSK						: STD_LOGIC;
	signal AESMainReset				: STD_LOGIC;
	signal AESMainEncrypt			: STD_LOGIC;
	signal StartXY						: STD_LOGIC;
	signal AEStopReset				: STD_LOGIC;
	signal AESbotReset				: STD_LOGIC;
	signal AESbotOrFeedback			: STD_LOGIC;
	signal OutputOrTau				: STD_LOGIC;
	signal FeedbackRegClear			: STD_LOGIC;
	signal FeedbackRegEnable		: STD_LOGIC;
	signal LoopEnable					: STD_LOGIC;
	signal AESMainDone				: STD_LOGIC;
	signal AEStopDone					: STD_LOGIC;
	signal AESbotDone					: STD_LOGIC;
				
	TYPE STATES_FSM IS (RESET_STATE, RECEIVE_COMMAND, 
							  SET_KEYS, SET_KEYS2, 
							  PROCESS_HEADER, PROCESS_HEADER2, 
							  DO_ENC_DEC, DO_ENC_DEC2, DO_ENC_DEC3, DO_ENC_DEC4, 
							  GenVer_TAG, GenVer_TAG2, GenVer_TAG3, 
							  DONE); 

	SIGNAL state, next_state : STATES_FSM;

	signal CounterEnable				: STD_LOGIC;
	signal CounterReset				: STD_LOGIC;
	signal CounterValue				: STD_LOGIC_VECTOR(1 downto 0);

	signal HeaderStartedIn			: STD_LOGIC;
	signal HeaderStarted				: STD_LOGIC;
	signal HeaderDoneIn				: STD_LOGIC;
	signal HeaderDone					: STD_LOGIC;

	signal EncryptStartedIn			: STD_LOGIC;
	signal EncryptStarted			: STD_LOGIC;

	signal DecryptStartedIn			: STD_LOGIC;
	signal DecryptStarted			: STD_LOGIC;

	signal IsLastIn					: STD_LOGIC;
	signal IsLast						: STD_LOGIC;
	
begin

	MainModuleIns: MainModule
   PORT MAP (  
		input     	     			=> input,     	     
		clk		     	  			=> clk,		     
		ForcedInputBits  			=> ForcedInputBits,     	     
		SaveK      	     			=> SaveK,      	     
		SaveK10      	  	   	=> SaveK10,      	     
		SaveL      	     			=> SaveL,      	     
		SaveKF  	     				=> SaveKF,  	     
		SaveTau    	     			=> SaveTau,     	     
		ProcessLStart	     		=> ProcessLStart,	     
		ProcessLEnable	   	  	=> ProcessLEnable,	     
		ProcessLClear	  	   	=> ProcessLClear,	     
		BlockCounterEnable		=> BlockCounterEnable,	     
		BlockCounterReset	  		=> BlockCounterReset,	     
		InputOrMLength	     		=> InputOrMLength,	     
		AEStopOrL	     			=> AEStopOrL,	     
		KOrSK		     				=> KOrSK,		     
		AESMainReset	     		=> AESMainReset,	     
		AESMainEncrypt				=> AESMainEncrypt,
		StartXY		     			=> StartXY,		     
		AEStopReset	     			=> AEStopReset,	     
		AESbotReset	     			=> AESbotReset,	     
		AESbotOrFeedback     	=> AESbotOrFeedback,     
		OutputOrTau	     			=> OutputOrTau,	     
		FeedbackRegClear    	 	=> FeedbackRegClear,     
		FeedbackRegEnable   		=> FeedbackRegEnable,    
		LoopEnable	     			=> LoopEnable,	     
		AESMainDone	     			=> AESMainDone,	     
		AEStopDone	     			=> AEStopDone,	     
		AESbotDone	     			=> AESbotDone,	
		output 		     			=> output);

	
	CounterIns: counter
	GENERIC MAP ( size => 2)
	PORT MAP ( 
		clk   => clk,
		en    => CounterEnable,
		reset	=> CounterReset,
		q 		=> CounterValue);
	
	---------------------------------
	
	state_Change: PROCESS(clk, reset)
	BEGIN
		IF (RISING_EDGE(clk)) THEN
			IF (reset = '1') THEN
				state <= RESET_STATE;
			ELSE
				state <= next_state;
			END IF;
						
			HeaderStarted		<= HeaderStartedIn;
			HeaderDone			<= HeaderDoneIn;
			EncryptStarted		<= EncryptStartedIn;
			DecryptStarted		<= DecryptStartedIn;
			IsLast				<= IsLastIn;
		END IF;
	END PROCESS;
			
	---------------------------------

	FSM : PROCESS (state, setkey, header, encrypt, decrypt, HeaderStarted, HeaderDone, EncryptStarted, DecryptStarted, AESMainDone, AEStopDone, AESbotDone, last, CounterValue, IsLast, ls, lt)
	BEGIN
		ForcedInputBits  			<= "0000";
		SaveK      	     			<= '0';
		SaveK10    	     			<= '0';
		SaveL      	     			<= '0';
		SaveKF  	     				<= '0';
		SaveTau    	     			<= '0';
		ProcessLStart	     		<= '0';
		ProcessLEnable	     		<= '0';
		ProcessLClear	     		<= '0';
		BlockCounterEnable		<= '0';
		BlockCounterReset	  		<= '0';
		InputOrMLength	     		<= '0';
		AEStopOrL	     			<= '0';
		KOrSK		     				<= '0';
		AESMainReset	     		<= '0';
		AESMainEncrypt				<= '0';
		StartXY		     			<= '0';
		AEStopReset	     			<= '0';
		AESbotReset	     			<= '0';
		AESbotOrFeedback    	 	<= '0';
		OutputOrTau	     			<= '0';
		FeedbackRegClear 	   	<= '0';
		FeedbackRegEnable	 	   <= '0';
		LoopEnable	     			<= '0';
		CounterEnable				<= '0';
		CounterReset				<= '0';
		
		HeaderStartedIn			<= HeaderStarted;
		HeaderDoneIn				<= HeaderDone;
		EncryptStartedIn			<= EncryptStarted;
		DecryptStartedIn			<= DecryptStarted;
		IsLastIn						<= IsLast;
		
		ready							<= '0';
		next_state 					<= state;

		CASE state IS
			WHEN RESET_STATE =>			
				BlockCounterReset					<= '1';
				HeaderStartedIn					<= '0';
				HeaderDoneIn						<= '0';
				EncryptStartedIn					<= '0';
				DecryptStartedIn					<= '0';
				next_state							<= RECEIVE_COMMAND;
				
			WHEN RECEIVE_COMMAND =>
				ready									<= '1';
				
				IF setkey = '1' THEN
					ProcessLClear				<= '1';
					CounterReset				<= '1';
					next_state					<= SET_KEYS;
				--------------------------------------------
				ELSIF header = '1' THEN
					IsLastIn						<= last;
					CounterReset				<= '1';
					next_state					<= PROCESS_HEADER;

					IF HeaderStarted = '0' THEN
						ProcessLStart	     	<= '1';
						ProcessLEnable	     	<= '1';
						FeedbackRegClear		<= '1';
					END IF;
				--------------------------------------------
				ELSIF (encrypt = '1') OR (decrypt = '1') THEN
					IsLastIn						<= last;
					BlockCounterEnable		<= '1';
					AESbotReset					<= '1';
					
					next_state					<= DO_ENC_DEC;

					IF (EncryptStarted = '0') AND (DecryptStarted = '0') THEN
						AESMainEncrypt			<= encrypt;
						StartXY 					<= '1';
						AEStopReset				<= '1';
						FeedbackRegClear		<= '1';
						ProcessLClear			<= '1';
						EncryptStartedIn		<= encrypt;
						DecryptStartedIn		<= decrypt;
					END IF;
				END IF;
								
			-------------------------------------------------
				
			WHEN SET_KEYS =>
				ForcedInputBits	<= "00" & CounterValue;
				InputOrMLength		<= '1';
				AEStopOrL			<= '1';
				
				IF CounterValue /= "11" THEN
					KOrSK				<= '1';
				END IF;
				
				AESMainReset		<= '1';
				AESMainEncrypt		<= '1';
				next_state			<= SET_KEYS2;
			
			WHEN SET_KEYS2 =>
				IF AESMainDone = '1' THEN
					CounterEnable			<= '1';
					next_state				<= SET_KEYS;
					
					IF CounterValue = "00" THEN
						SaveK					<= '1';
					ELSIF CounterValue = "01" THEN
						SaveL					<= '1';
					ELSIF CounterValue = "10" THEN
						SaveKF				<= '1';
					ELSE   --IF CounterValue = "11" THEN
						SaveK10				<= '1';
						next_state			<= RECEIVE_COMMAND;
					END IF;
				END IF;

			-------------------------------------------------

			WHEN PROCESS_HEADER =>
				IF (HeaderStarted = '0') OR (CounterValue(0) = '1') OR (CounterValue(1) = '1') THEN
					InputOrMLength			<= '1';
				END IF;
								
				ForcedInputBits(2)		<= (NOT HeaderStarted) AND ls;
				ForcedInputBits(3)		<= CounterValue(0) OR ((NOT HeaderStarted) AND lt);
				LoopEnable					<= CounterValue(1);
				
				AEStopOrL	     			<= '1';
				AESMainReset				<= '1';
				AESMainEncrypt				<= '1';
				next_state					<= PROCESS_HEADER2;
								
			WHEN PROCESS_HEADER2 =>
				AESbotOrFeedback     	<= '1';
				
				IF AESMainDone	= '1' THEN
					FeedbackRegEnable		<=	'1';
					
					IF CounterValue = "00" THEN
						ProcessLEnable	   <= '1';
					ELSE
						ProcessLClear	   <= '1';
					END IF;
					
					
					HeaderStartedIn		<= '1';
					
					IF HeaderStarted = '0' THEN
						next_state			<= PROCESS_HEADER;
					ELSE	
						IF IsLast = '1' THEN
							CounterEnable		<= '1';
							
							IF CounterValue(1) = '1' THEN
								SaveTau					<= '1';
								HeaderDoneIn			<= '1';							
								next_state				<= RECEIVE_COMMAND;
							ELSE
								next_state			<= PROCESS_HEADER;
							END IF;
						ELSE
							next_state			<= RECEIVE_COMMAND;
						END IF;
					END IF;
				END IF;

			--------------------------------------------------

			WHEN DO_ENC_DEC =>
				IF AEStopDone = '1' THEN
					IF IsLast = '1' THEN
						InputOrMLength		<= '1';
						AEStopOrL			<= '1';
						AESMainReset		<= '1';
						AESMainEncrypt		<= '1';
						next_state			<= DO_ENC_DEC2;
					ELSE
						AEStopReset			<= '1';
						AESMainReset		<= '1';
						AESMainEncrypt		<= EncryptStarted;
						next_state			<= DO_ENC_DEC4;
					END IF;
				END IF;

			WHEN DO_ENC_DEC2 =>
				IF AESMainDone = '1' THEN
					AESbotOrFeedback		<= '1';
					FeedbackRegEnable		<= '1';
					next_state				<= DO_ENC_DEC3;
				END IF;

			WHEN DO_ENC_DEC3 =>
					LoopEnable				<= '1';
					AEStopReset				<= '1';
					AESMainReset			<= '1';
					AESMainEncrypt			<= EncryptStarted;
					next_state				<= DO_ENC_DEC4;

			WHEN DO_ENC_DEC4 =>					
				IF (AEStopDone AND AESMainDone AND AESbotDone) = '1' THEN
					ready 							<= '1';
					
					IF IsLast = '1' THEN
						next_state					<= GenVer_TAG;	
					ELSE
						IF (encrypt = '1') OR (decrypt = '1') THEN					
							IsLastIn					<= last;
							BlockCounterEnable	<= '1';
							AESbotReset				<= '1';
					
							next_state				<= DO_ENC_DEC;
						ELSE
							next_state				<= RECEIVE_COMMAND;
						END IF;	
					END IF;
				END IF;

			--------------------------------------------------

			WHEN GenVer_TAG =>
				ready							<= '1';
				
				OutputOrTau					<= '1';
				FeedbackRegEnable			<= '1';
				BlockCounterReset			<= '1';				
				next_state					<= GenVer_TAG2;
				
			WHEN GenVer_TAG2 =>
				InputOrMLength				<= EncryptStarted;
				LoopEnable					<= '1';
				AESMainReset				<= '1';
				AESMainEncrypt				<= EncryptStarted;
				AESbotReset					<= '1';
				next_state					<= GenVer_TAG3;

			WHEN GenVer_TAG3 =>
				IF (AESMainDone AND AESbotDone) = '1' THEN
					next_state				<= DONE;
				END IF;

			--------------------------------------------------

			WHEN DONE =>
				ready							<= '1';
				
		END CASE;
	END PROCESS;
	
end Behavioral;

