----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    17:05:13 12/31/2014 
-- Module Name:    Process_L - Behavioral 
-- Project Name:   POET 2
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Process_L is
    Port ( input 		: in  STD_LOGIC_VECTOR (127 downto 0);
           clk 		: in  STD_LOGIC;
           start 		: in  STD_LOGIC;
           enable 	: in  STD_LOGIC;
           clear 		: in  STD_LOGIC;
           output 	: out  STD_LOGIC_VECTOR (127 downto 0));
end Process_L;

architecture Behavioral of Process_L is

	COMPONENT reg_FDER IS
	GENERIC ( size: POSITIVE := 128);
	PORT ( clk  : IN  STD_LOGIC;
			 D    : IN  STD_LOGIC_VECTOR ((size-1) DOWNTO 0);
			 clr  : IN  STD_LOGIC;
			 en   : IN  STD_LOGIC;
			 Q    : OUT STD_LOGIC_VECTOR ((size-1) DOWNTO 0));
	END COMPONENT;
		
	COMPONENT GF128Mul2 is
	PORT ( input  : in  STD_LOGIC_VECTOR (127 downto 0);
			 output : out STD_LOGIC_VECTOR (127 downto 0));
	end COMPONENT;

	-----------------------------

	signal RegInput 		: STD_LOGIC_VECTOR (127 DOWNTO 0);
	signal RegOutput 		: STD_LOGIC_VECTOR (127 DOWNTO 0);
	signal Feedback 		: STD_LOGIC_VECTOR (127 DOWNTO 0);

begin

	RegInput <= input WHEN start = '1' ELSE Feedback;

	LReg: reg_FDER
	GENERIC MAP (size => 128)
	PORT MAP (
		clk	=> clk,
		D		=> RegInput,
		clr	=> clear,
		en		=> enable,
		Q		=> RegOutput);	
	
	output <= RegOutput;
	
	GF128Mul2Ins1: GF128Mul2
	PORT MAP (
		input		=> RegOutput,
		output	=> Feedback);	

end Behavioral;

