----------------------------------------------------------------------------------
-- Ruhr-University Bochum, Germany
-- Amir Moradi (amir.moradi@rub.de)
-- 
-- Last Update:    09:11:46 03/01/2016
-- VHDL Test Bench for module: POET_Full
-- Project Name:   POET 2
----------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
ENTITY test_Full_POET IS
END test_Full_POET;
 
ARCHITECTURE behavior OF test_Full_POET IS 
 

    COMPONENT POET_Full
    PORT(
         input : IN  std_logic_vector(127 downto 0);
         last : IN  std_logic;
         clk : IN  std_logic;
			ls			: in  STD_LOGIC;
			lt			: in	STD_LOGIC;	
         ready : OUT  std_logic;
         reset : IN  std_logic;
         setkey : IN  std_logic;
         header : IN  std_logic;
         encrypt 	: in  STD_LOGIC;
         decrypt 	: in  STD_LOGIC;
         output : OUT  std_logic_vector(127 downto 0)
        );
    END COMPONENT;
    

   --Inputs
   signal input : std_logic_vector(127 downto 0) := (others => '0');
   signal last : std_logic := '0';
   signal clk : std_logic := '0';
   signal ls : std_logic := '1';
   signal lt : std_logic := '1';	
   signal reset : std_logic := '0';
   signal setkey : std_logic := '0';
   signal header : std_logic := '0';
   signal encrypt : std_logic := '0';
   signal decrypt : std_logic := '0';

 	--Outputs
   signal ready : std_logic;
   signal output : std_logic_vector(127 downto 0);

   -- Clock period definitions
   constant clk_period : time := 1 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: POET_Full PORT MAP (
          input => input,
          last => last,
          clk => clk,
			 ls	=> ls,
			 lt	=> lt,
          ready => ready,
          reset => reset,
          setkey => setkey,
          header => header,
          encrypt => encrypt,
          decrypt => decrypt,
          output => output
        );

   -- Clock process definitions
   clk_process :process
   begin
		clk <= '0';
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
   end process;
  

   -- Stimulus process
   stim_proc: process
   begin		

    wait for 10 ns;	

		reset	<= '1';

      wait for clk_period;

		reset 	<= '0';
		
      wait for clk_period;
		
		setkey 	<= '1';
		input		<= x"00112233445566778899aabbccddeeff";  -- Secret Key (SK)
		
      wait until ready = '0';
		wait for clk_period/2;
		setkey 	<= '0';
		
		wait until ready = '1';
		wait for clk_period/2;
				
		header		<= '1';
		input			<= x"0102030405060708090a0b0c0d0e0f10"; -- header
		last			<= '1';
		
      wait until ready = '0';
		wait for clk_period/2;
		
		header 	<= '0';
		
		wait until ready = '1';
		wait for clk_period/2;

		encrypt		<= '1';
		input			<= x"000102030405060708090a0b0c0d0e0f"; -- plaintext1
		last		   <= '0';
		
      wait until ready = '0';
		wait for clk_period/2;
		
		encrypt 		<= '0';

		wait until ready = '1';  -- output should be (ciphertext1): fb4902581199c00d44d29714ac112552
		wait for clk_period/2;

		encrypt		<= '1';
		input			<= x"101112131415161718191a1b1c1d1e1f"; -- plaintext2
		
      wait until ready = '0';
		wait for clk_period/2;
		
		encrypt 		<= '0';

		wait until ready = '1'; -- output should be (ciphertext2): 7408d8a9b3f2f1d80cac69ca9a43dfc5
		wait for clk_period/2;

		encrypt		<= '1';
		input			<= x"202122232425262728292a2b2c2d2e2f"; -- plaintext3
		last			<= '1';
		
      wait until ready = '0';
		wait for clk_period/2;
		
		encrypt 		<= '0';

		wait until ready = '1'; -- output should be (ciphertext3): f178088287875112a79904947371e82b
		wait for clk_period/2;


      wait until ready = '0';
		wait for clk_period/2;

		wait until ready = '1'; -- output should be (tag): c6e1d7067f1cef4c613f47cb8cc3c89a
		wait for clk_period/2;


		--****************************************

      wait for 20 ns;	

		reset	<= '1';

      wait for clk_period;

		reset 	<= '0';
		
      wait for clk_period;
						
		header		<= '1';
		input			<= x"0102030405060708090a0b0c0d0e0f10"; -- header
		last			<= '1';
		
      wait until ready = '0';
		wait for clk_period/2;
		
		header 	<= '0';
		
		wait until ready = '1';
		wait for clk_period/2;

		decrypt		<= '1';
		input			<= x"fb4902581199c00d44d29714ac112552"; -- ciphertext1
		last			<= '0';

      wait until ready = '0';
		wait for clk_period/2;
		
		decrypt 		<= '0';

		wait until ready = '1'; -- output should be (plaintext1): 000102030405060708090a0b0c0d0e0f
		wait for clk_period/2;

		decrypt		<= '1';
		input			<= x"7408d8a9b3f2f1d80cac69ca9a43dfc5"; -- ciphertext2
		
      wait until ready = '0';
		wait for clk_period/2;
		
		decrypt 		<= '0';

		wait until ready = '1'; -- output should be (plaintext2): 101112131415161718191a1b1c1d1e1f
		wait for clk_period/2;

		decrypt		<= '1';
		input			<= x"f178088287875112a79904947371e82b"; -- ciphertext3
		last			<= '1';
		
      wait until ready = '0';
		wait for clk_period/2;
		
		decrypt 		<= '0';
	
		wait until ready = '1'; -- output should be (plaintext3): 202122232425262728292a2b2c2d2e2f
		wait for clk_period/2;
		
		input			<= x"c6e1d7067f1cef4c613f47cb8cc3c89a"; -- tag

		wait until ready = '0';
		wait for clk_period/2;
		
		wait until ready = '1'; -- output should be: 00000000000000000000000000000000
		wait for clk_period/2;


      wait;
   end process;

END;
